@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  * 
- * $Id: disk_info.c,v 1.15 2004/04/08 20:13:44 nold Exp $
+ * $Id: disk_info.c,v 1.16 2010/07/25 00:44:52 nold Exp $
  *
  */
 
@@ -214,7 +214,7 @@ void disk_info_setup(disk_info_t di, tolist_t tohead)
 	if (!di)
 		return;
 
-	di->spindown = spindown_setup(di->major, di->minor);	
+	di->spindown = spindown_setup(di);	
 	if (!di->spindown) {
 		ERR("Don't know how to spindown %s", di->name);
 		return;
