@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  * 
- * $Id: intr_stat.c,v 1.6 2003/10/12 15:32:36 nold Exp $
+ * $Id: intr_stat.c,v 1.7 2005/05/08 22:17:51 nold Exp $
  *
  */
 
@@ -243,7 +243,7 @@ int intr_stat_register(int n)
 	}
 	irq->active = 1;
 	global.nb_watch++;
-	INFO("Monitoring interrupt #%d \"%s\"", n,
+	DEBUG("Monitoring interrupt #%d \"%s\"", n,
 	     irq->name ? irq->name : UNKNOWN_INTERRUPT);
 	return 1;
 }
