@@ -3,7 +3,7 @@
  * 
  * (C) 2000 Daniel Kobras <kobras@linux.de>
  * 
- * $Id: spindown.h,v 1.2 2000/10/01 17:28:42 belbo Exp $
+ * $Id: spindown.h,v 1.3 2010/07/25 00:44:52 nold Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 
 typedef int(*spindown_t)(int);
 
-spindown_t spindown_setup(int major, int minor);
+spindown_t spindown_setup(disk_info_t di);
 int spindown(disk_info_t di);
 
 #endif
