@@ -10,7 +10,7 @@
  * 
  * Pavel Machek's work was sponsored by SuSE.
  * 
- * $Id: state.c,v 1.27 2004/04/08 20:13:44 nold Exp $
+ * $Id: state.c,v 1.30 2010/07/25 00:59:26 nold Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -150,17 +150,121 @@ void devname_init(void)
 	set_path_fixup();
 }
 
+/* In the days of udev, closing a block device that had been open for writing
+ * triggers an udev event to re-read the partition table (presumably). The
+ * read comes in asynchronously after the close(). In other words, the old
+ * implementation of sync_part() below synched a device, closed the fds, and
+ * thus triggered a read that started in parallel when we tried to issue the
+ * spindown, causing an immediate spinup. This quick&dirty hack caches the
+ * writable fd between calls to sync_part() so we can avoid the close()
+ * without leaking fds. There are certainly saner places to cache this info,
+ * but I was lazy.
+ */
+typedef struct fdcache fdcache_t;
+struct fdcache {
+	fdcache_t *next;
+	char *name;
+	int fd;
+};
+
+static fdcache_t *head = NULL;
+
+static void fdcache_add(char *name, int fd)
+{
+	fdcache_t *n;
+
+	n = malloc(sizeof(fdcache_t));
+	if (!n)
+		return;
+
+	n->name = strdup(name);
+	if (!n->name) {
+		free(n);
+		return;
+	}
+
+	n->fd = fd;
+
+	if (head)
+		n->next = head->next;
+	else
+		head = n;
+
+	head->next = n;
+	head = n;
+
+	DEBUG("Add entry for %s (fd %d)", name, fd);
+}
+
+static int fdcache_lookup(char *name)
+{
+	fdcache_t *n;
+
+	if (!head)
+		return -1;
+
+	n = head;
+
+	do {
+		if (!strcmp(n->name, name)) {
+			/* Optimize the most likely next lookup. */
+			head = n->next;
+			DEBUG("Found cached fd for %s (fd %d)", name, n->fd);
+			return n->fd;
+		}
+		n = n->next;
+	} while (n != head);
+
+	return -1;
+}
+
+static void fdcache_close(void)
+{
+	fdcache_t *n, *next;
+
+	if (!head)
+		return;
+
+	n = head;
+
+	do {
+		close(n->fd);
+		next = n->next;
+		free(n);
+		n = next;
+	} while (n != head);
+}
+
+static int fdcache_open(char *name, int mode) {
+	int fd;
+
+	fd = fdcache_lookup(name);
+	if (fd != -1)
+		return fd;
+
+	fd = open(name, mode);
+	if (fd == -1)
+		return -1;
+
+	fdcache_add(name, fd);
+
+	return fd;
+}	
+
 static int sync_part(char *name)
 {
 	int fd, ret=0;
 
 	DEBUG("Syncing %s", name);
-	fd = open(name, O_WRONLY);
+	fd = fdcache_open(name, O_WRONLY);
 	if (fd==-1)
 		return 0;
 	if (!fsync(fd))
 		ret=1;
-	close(fd);
+	/* Closing a writable fd of a block device causes a re-read via udev.
+	 * This could interfere with a spindown request. Therefore, we use
+	 * the fdcache and skip the close(). */
+	/* close(fd); */
 	return ret;
 }
 
@@ -195,8 +299,10 @@ static void sync_disk(disk_info_t di)
 		return;
 	}
 	
-	if (!sync_current_disk())
+	if (!sync_current_disk()) {
+		INFO("%s: sync failed, marking read-only", di->name);
 		disk_info_mark_ro(di);
+	}
 }
 		
 static void sync_spinning_disks(disk_info_t head)
@@ -272,14 +378,14 @@ static void advance_timeouts(disk_info_t di)
  */
 static int try_spindown(disk_stat_t ds, disk_info_t di)
 {
-	
+
 	DEBUG("Disk %s, Time left %d.", di->name, di->time_left);
 
 	if (di->time_left > 0)
 		/* Spindown time not reached. */
 		return 0;
 
-	INFO("Spinning down %s.", di->name);
+	DEBUG("Spinning down %s.", di->name);
 
 	/* Syncing can last a while, in which time new data might have been
 	 * produced. So we sync twice, assuming that the second sync is
@@ -289,7 +395,7 @@ static int try_spindown(disk_stat_t ds, disk_info_t di)
 		sync_disk(di);
 		sync_disk(di);
 	}
-	
+
 	disk_stat_update(ds);
 	
 	/* Cancel spindown if there was other activity than our sync on the
@@ -436,6 +542,16 @@ void nfd_daemon(disk_info_t head, disk_stat_t stat)
 			if (sync_left <= 0) {
 				sync_spinning_disks(head);
 				sync_left = kupdate_get_interval(kupdate);
+				/* Tricky case: Someone else has shut down
+				 * kupdate/pdflush from under us. We're not
+				 * turning it back on in this case, but we're
+				 * still running our own syncing routine at a
+				 * default 5 second interval. I'm not quite
+				 * sure that this is the behaviour users
+				 * expect here, though.
+				 */
+				if (sync_left <= 0)
+					sync_left = 5;
 			}
 			left = get_min_timeout(head)/60;
 			if (sync_left < left)
